/* PS/2 Mouse to Classic Mac Converter
 * Interfaces a PS2 mouse and outputs the standard quadrature outputs
 * the classic (pre-ADB) Macs require.
 * Copyright 2012 Giacomo Ferrari
 * http://giferrari.net/blog
 * Licensed under the TAPR Open Hardware License (www.tapr.org/OHL)
 * If running with an ATTINY84/44, make sure the clock is 8Mhz!
 * Otherwise, we can't keep up with the real mouse.
 * Requires: PS2mouse for the Arduino, found here: http://arduino.cc/playground/componentLib/Ps2mouse
 * If you want to run this on an ATtiny, install the ATtiny support for Arduino from here: https://github.com/damellis/attiny
 */

// PS2mouse library, see http://arduino.cc/playground/componentLib/Ps2mouse
#include <ps2.h>

#define PIN_HORIZ_A 9
#define PIN_HORIZ_B 7
#define PIN_VERT_A  0
#define PIN_VERT_B  8
#define PIN_BUTTON 10

#define PIN_LED 5

#define PIN_REAL_MOUSE_CLK 2
#define PIN_REAL_MOUSE_DTA 3

const uint8_t QuadratureEncoding[4] =
{
    0b00,
    0b01,
    0b11,
    0b10
};

PS2 realMouse(PIN_REAL_MOUSE_CLK, PIN_REAL_MOUSE_DTA);

uint8_t g_horizQuadStep = 0;
uint8_t g_vertQuadStep = 0;

void writeVerticalQuadratureOutputs();
void writeHorizontalQuadratureOutputs();

void mouseInit()
{
  realMouse.write(0xff);  // RST
  digitalWrite(PIN_LED, LOW);
  realMouse.read();
  realMouse.read();
  realMouse.read();
  realMouse.write(0xf0); // Remote.
  realMouse.read();
  delayMicroseconds(100);
}

void setup()
{
  pinMode(PIN_HORIZ_A, OUTPUT);
  pinMode(PIN_HORIZ_B, OUTPUT);
  pinMode(PIN_VERT_A, OUTPUT);
  pinMode(PIN_VERT_B, OUTPUT);
  pinMode(PIN_BUTTON, OUTPUT);
  pinMode(PIN_LED, OUTPUT);

  delay(100);
  
  cli(); // Disable interrupts, they interfere with our timing.
  
  digitalWrite(PIN_LED, HIGH);
  mouseInit();
  digitalWrite(PIN_LED, LOW);
}

void loop()
{
  char mstat;
  char mx;
  char my;
  boolean goingRight;
  boolean goingDown;
  
  realMouse.write(0xeb);
  realMouse.read();
  mstat = realMouse.read();
  mx = realMouse.read();
  my = realMouse.read();

  goingRight = mx > 0;
  goingDown = my > 0;
  mx = abs(mx);
  my = abs(my);
  
  while (mx > 0)
  {
    g_horizQuadStep += goingRight ? 1 : -1;
    writeHorizontalQuadratureOutputs();
    delayMicroseconds(100);
    mx --;
  }
  
  while (my > 0)
  {
    g_vertQuadStep += goingDown ? 1 : -1;
    writeVerticalQuadratureOutputs();
    delayMicroseconds(100);
    my --;
  }
  
  digitalWrite(PIN_BUTTON, !(mstat & 0b11));
  digitalWrite(PIN_LED, mstat & 0b11);
}

void writeHorizontalQuadratureOutputs()
{
  uint8_t quadEncoding = QuadratureEncoding[g_horizQuadStep % 4];
  digitalWrite(PIN_HORIZ_A, quadEncoding & 1);
  digitalWrite(PIN_HORIZ_B, quadEncoding>>1 & 1);
}

void writeVerticalQuadratureOutputs()
{
  uint8_t quadEncoding = QuadratureEncoding[g_vertQuadStep % 4];
  digitalWrite(PIN_VERT_A, quadEncoding & 1);
  digitalWrite(PIN_VERT_B, quadEncoding>>1 & 1);
}
